﻿local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Broker_LFD")
local LFD = CreateFrame("Frame", "Broker_LFD")

LFD.obj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_LFD", {type = "data source", icon = "Interface\\AddOns\\Broker_LFD\\Inactive", label = "LFD"})

LFD:RegisterEvent("LFG_UPDATE")
LFD:RegisterEvent("LFG_QUEUE_STATUS_UPDATE")
LFD:RegisterEvent("PLAYER_ENTERING_WORLD")
LFD:RegisterEvent("PARTY_MEMBERS_CHANGED")
LFD:RegisterEvent("LFG_PROPOSAL_SHOW")
LFD:RegisterEvent("LFG_PROPOSAL_UPDATE")

local statusFrame = LFDSearchStatus
local tankWait, healerWait, damageWait, myWait

local function statusChar(char, status, need)
	if status and need and status < need then
		return ("|cff00ff00%s|r"):format(char)
	else
		return ("|cffff0000%s|r"):format(char)
	end
end

LFD:SetScript("OnEvent", function(self, event)
	local mode, submode = GetLFGMode()
	if mode then
		LFD.obj.icon = "Interface\\AddOns\\Broker_LFD\\Active"
	else
		LFD.obj.icon = "Interface\\AddOns\\Broker_LFD\\Inactive"
	end
	statusFrame:SetParent(self)
	statusFrame:SetFrameStrata("TOOLTIP")
	statusFrame:SetClampedToScreen(true)
	MiniMapLFGFrame:Hide()
	
	if mode == "queued" then
		local hasData,  leaderNeeds, tankNeeds, healerNeeds, dpsNeeds, instanceType, instanceName, averageWait = GetLFGQueueStats()
		tankWait, healerWait, damageWait, myWait = select(9, GetLFGQueueStats())
		
		LFD.obj.text = ("%s%s%s%s%s"):format(
			statusChar(L["T"], tankNeeds, 1),
			statusChar(L["H"], healerNeeds, 1),
			statusChar(L["D"], dpsNeeds, 3),
			statusChar(L["D"], dpsNeeds, 2),
			statusChar(L["D"], dpsNeeds, 1)
		)
	else
		LFD.obj.text = ""
	end
end)

local function colorFor(wait)
	local r, g, b = 0, 0, 0
	if wait > 120 then
		r = 255
		g = (300 - math.min(wait, 300)) / 180 * 255
	else
		g = 255
		r = math.min(wait, 120) / 120 * 255
	end		
	return ("%02x%02x%02x"):format(r,g,b)
end

function LFD.obj.OnTooltipShow(self)
	local mode, submode = GetLFGMode()
	if ( mode == "queued" ) then
		statusFrame:ClearAllPoints()
		statusFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 100)
		statusFrame:Show()

		if tankWait then
			local tl = tankWait > 60 and "m" or "s"
			local hl = healerWait > 60 and "m" or "s"
			local dl = damageWait > 60 and "m" or "s"
			local tw = tankWait > 60 and tankWait / 60.0 or tankWait
			local hw = healerWait > 60 and healerWait / 60.0 or healerWait
			local dw = damageWait > 60 and damageWait / 60.0 or damageWait
			GameTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			GameTooltip:ClearLines()
			GameTooltip:SetText(AVERAGE_WAIT_TIME)
			GameTooltip:AddDoubleLine(TANK, ("|cff%s%2.1f%s|r"):format(colorFor(tankWait), tw, tl))
			GameTooltip:AddDoubleLine(HEALER, ("|cff%s%2.1f%s|r"):format(colorFor(healerWait), hw, hl))
			GameTooltip:AddDoubleLine(DAMAGE, ("|cff%s%2.1f%s|r"):format(colorFor(damageWait), dw, dl))
			GameTooltip:Show()
		end
	elseif ( mode == "proposal" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(DUNGEON_GROUP_FOUND_TOOLTIP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(CLICK_HERE_FOR_MORE_INFO, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "rolecheck" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(ROLE_CHECK_IN_PROGRESS_TOOLTIP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "listed" ) then
		GameTooltip:SetText(LOOKING_FOR_RAID)
		GameTooltip:AddLine(YOU_ARE_LISTED_IN_LFR, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	elseif ( mode == "lfgparty" ) then
		GameTooltip:SetText(LOOKING_FOR_DUNGEON)
		GameTooltip:AddLine(YOU_ARE_IN_DUNGEON_GROUP, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1)
		GameTooltip:Show()
	end	
end

function LFD.obj.OnClick(self, button)
	local mode, submode = GetLFGMode()
	if ((mode and button == "RightButton") or mode == "lfgparty" or mode == "abandonedInDungeon") then
		statusFrame:Hide()
		MiniMapLFGFrameDropDown.point = "TOPRIGHT"
		MiniMapLFGFrameDropDown.relativePoint = "BOTTOMLEFT"
		ToggleDropDownMenu(1, nil, MiniMapLFGFrameDropDown, "cursor", 0, 0)
	elseif mode == "proposal" then
		if not LFDDungeonReadyPopup:IsShown() then
			PlaySound("igCharacterInfoTab")
			StaticPopupSpecial_Show(LFDDungeonReadyPopup)
		end
	elseif mode == "queued" or mode == "rolecheck" then
		ToggleLFDParentFrame()
	elseif mode == "listed" or (not mode and button == "RightButton") then
		ToggleLFRParentFrame()
	else
		ToggleLFDParentFrame()
	end
end

function LFD.obj.OnLeave(self)
	GameTooltip:Hide()
	statusFrame:Hide()
end
